@@ -1103,12 +1103,14 @@ tracker_db_manager_attach_database (TrackerDBManager    *db_manager,
 					g_propagate_error (error, inner_error);
 					return FALSE;
 				}
+
+				g_clear_error (&inner_error);
 			}
 		}
 	}
 
 	if (!tracker_db_interface_attach_database (iface, file, name, error)) {
-		g_object_unref (file);
+		g_clear_object (&file);
 		return FALSE;
 	}
 
