@@ -245,7 +245,6 @@ tracker_sparql_connection_query_finish (TrackerSparqlConnection  *connection,
  * tracker_sparql_connection_update:
  * @connection: a #TrackerSparqlConnection
  * @sparql: string containing the SPARQL update query
- * @priority: the priority for the operation
  * @cancellable: a #GCancellable used to cancel the operation
  * @error: #GError for error reporting.
  *
@@ -259,7 +258,6 @@ tracker_sparql_connection_query_finish (TrackerSparqlConnection  *connection,
 void
 tracker_sparql_connection_update (TrackerSparqlConnection  *connection,
                                   const gchar              *sparql,
-                                  gint                      priority,
                                   GCancellable             *cancellable,
                                   GError                  **error)
 {
@@ -270,7 +268,6 @@ tracker_sparql_connection_update (TrackerSparqlConnection  *connection,
 
 	return TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update (connection,
 	                                                                 sparql,
-	                                                                 priority,
 	                                                                 cancellable,
 	                                                                 error);
 }
@@ -279,7 +276,6 @@ tracker_sparql_connection_update (TrackerSparqlConnection  *connection,
  * tracker_sparql_connection_update_async:
  * @connection: a #TrackerSparqlConnection
  * @sparql: string containing the SPARQL update query
- * @priority: the priority for the asynchronous operation
  * @cancellable: a #GCancellable used to cancel the operation
  * @callback: user-defined #GAsyncReadyCallback to be called when
  *            asynchronous operation is finished.
@@ -290,7 +286,6 @@ tracker_sparql_connection_update (TrackerSparqlConnection  *connection,
 void
 tracker_sparql_connection_update_async (TrackerSparqlConnection *connection,
                                         const gchar             *sparql,
-                                        gint                     priority,
                                         GCancellable            *cancellable,
                                         GAsyncReadyCallback      callback,
                                         gpointer                 user_data)
@@ -301,7 +296,6 @@ tracker_sparql_connection_update_async (TrackerSparqlConnection *connection,
 
 	TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update_async (connection,
 	                                                                sparql,
-	                                                                priority,
 	                                                                cancellable,
 	                                                                callback,
 	                                                                user_data);
@@ -334,7 +328,6 @@ tracker_sparql_connection_update_finish (TrackerSparqlConnection  *connection,
  * @connection: a #TrackerSparqlConnection
  * @sparql: an array of strings containing the SPARQL update queries
  * @sparql_length: the amount of strings you pass as @sparql
- * @priority: the priority for the asynchronous operation
  * @cancellable: a #GCancellable used to cancel the operation
  * @callback: user-defined #GAsyncReadyCallback to be called when
  *            asynchronous operation is finished.
@@ -348,7 +341,6 @@ void
 tracker_sparql_connection_update_array_async (TrackerSparqlConnection  *connection,
                                               gchar                   **sparql,
                                               gint                      sparql_length,
-                                              gint                      priority,
                                               GCancellable             *cancellable,
                                               GAsyncReadyCallback       callback,
                                               gpointer                  user_data)
@@ -360,7 +352,6 @@ tracker_sparql_connection_update_array_async (TrackerSparqlConnection  *connecti
 	TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update_array_async (connection,
 	                                                                      sparql,
 	                                                                      sparql_length,
-	                                                                      priority,
 	                                                                      cancellable,
 	                                                                      callback,
 	                                                                      user_data);
@@ -426,7 +417,6 @@ tracker_sparql_connection_update_array_finish (TrackerSparqlConnection  *connect
  * tracker_sparql_connection_update_blank:
  * @connection: a #TrackerSparqlConnection
  * @sparql: string containing the SPARQL update query
- * @priority: the priority for the operation
  * @cancellable: a #GCancellable used to cancel the operation
  * @error: #GError for error reporting.
  *
@@ -443,7 +433,6 @@ tracker_sparql_connection_update_array_finish (TrackerSparqlConnection  *connect
 GVariant *
 tracker_sparql_connection_update_blank (TrackerSparqlConnection  *connection,
                                         const gchar              *sparql,
-                                        gint                      priority,
                                         GCancellable             *cancellable,
                                         GError                  **error)
 {
@@ -454,7 +443,6 @@ tracker_sparql_connection_update_blank (TrackerSparqlConnection  *connection,
 
 	return TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update_blank (connection,
 	                                                                       sparql,
-	                                                                       priority,
 	                                                                       cancellable,
 	                                                                       error);
 }
@@ -463,7 +451,6 @@ tracker_sparql_connection_update_blank (TrackerSparqlConnection  *connection,
  * tracker_sparql_connection_update_blank_async:
  * @connection: a #TrackerSparqlConnection
  * @sparql: string containing the SPARQL update query
- * @priority: the priority for the asynchronous operation
  * @cancellable: a #GCancellable used to cancel the operation
  * @callback: user-defined #GAsyncReadyCallback to be called when
  *            asynchronous operation is finished.
@@ -474,7 +461,6 @@ tracker_sparql_connection_update_blank (TrackerSparqlConnection  *connection,
 void
 tracker_sparql_connection_update_blank_async (TrackerSparqlConnection *connection,
                                               const gchar             *sparql,
-                                              gint                     priority,
                                               GCancellable            *cancellable,
                                               GAsyncReadyCallback      callback,
                                               gpointer                 user_data)
@@ -485,7 +471,6 @@ tracker_sparql_connection_update_blank_async (TrackerSparqlConnection *connectio
 
 	TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update_blank_async (connection,
 	                                                                      sparql,
-	                                                                      priority,
 	                                                                      cancellable,
 	                                                                      callback,
 	                                                                      user_data);
@@ -566,7 +551,6 @@ tracker_sparql_connection_query_statement (TrackerSparqlConnection  *connection,
 /**
  * tracker_sparql_connection_create_notifier:
  * @connection: a #TrackerSparqlConnection
- * @flags: flags to modify notifier behavior
  *
  * Creates a new #TrackerNotifier to notify about changes in @connection.
  * See #TrackerNotifier documentation for information about how to use this
@@ -576,12 +560,11 @@ tracker_sparql_connection_query_statement (TrackerSparqlConnection  *connection,
  *          when no longer needed.
  **/
 TrackerNotifier *
-tracker_sparql_connection_create_notifier (TrackerSparqlConnection *connection,
-                                           TrackerNotifierFlags     flags)
+tracker_sparql_connection_create_notifier (TrackerSparqlConnection *connection)
 {
 	g_return_val_if_fail (TRACKER_IS_SPARQL_CONNECTION (connection), NULL);
 
-	return TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->create_notifier (connection, flags);
+	return TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->create_notifier (connection);
 }
 
 /**
