@@ -224,14 +224,18 @@ write_cursor (QueryRequest          *request,
 	while (tracker_sparql_cursor_next (cursor, NULL, &inner_error)) {
 		glong cur_offset = -1;
 
-		g_data_output_stream_put_int32 (request->data_stream, n_columns, NULL, NULL);
+		if (!g_data_output_stream_put_int32 (request->data_stream, n_columns,
+		                                     NULL, &inner_error))
+			break;
 
 		for (i = 0; i < n_columns; i++) {
 			glong len;
 
-			g_data_output_stream_put_int32 (request->data_stream,
-			                                tracker_sparql_cursor_get_value_type (cursor, i),
-			                                NULL, NULL);
+			if (!g_data_output_stream_put_int32 (request->data_stream,
+			                                     tracker_sparql_cursor_get_value_type (cursor, i),
+			                                     NULL, &inner_error))
+				goto out;
+
 			values[i] = tracker_sparql_cursor_get_string (cursor, i, &len);
 			len++;
 			cur_offset += len;
@@ -239,18 +243,25 @@ write_cursor (QueryRequest          *request,
 		}
 
 		for (i = 0; i < n_columns; i++) {
-			g_data_output_stream_put_int32 (request->data_stream,
-			                                offsets[i], NULL, NULL);
+			if (!g_data_output_stream_put_int32 (request->data_stream,
+			                                     offsets[i], NULL, &inner_error))
+				goto out;
 		}
 
 		for (i = 0; i < n_columns; i++) {
-			g_data_output_stream_put_string (request->data_stream,
-			                                 values[i] ? values[i] : "",
-			                                 NULL, NULL);
-			g_data_output_stream_put_byte (request->data_stream, 0, NULL, NULL);
+			if (!g_data_output_stream_put_string (request->data_stream,
+			                                      values[i] ? values[i] : "",
+			                                      NULL, &inner_error))
+				goto out;
+
+			if (!g_data_output_stream_put_byte (request->data_stream, 0, NULL,
+			                                    &inner_error))
+				goto out;
 		}
 	}
 
+out:
+
 	g_free (values);
 	g_free (offsets);
 
@@ -262,31 +273,50 @@ write_cursor (QueryRequest          *request,
 	}
 }
 
-/* Takes ownership on both cursor and error */
 static void
-handle_cursor_reply (QueryRequest        *request,
-                     TrackerSparqlCursor *cursor,
-                     GError              *error)
+handle_cursor_reply (GTask        *task,
+                     gpointer      source_object,
+                     gpointer      task_data,
+                     GCancellable *cancellable)
 {
+	TrackerSparqlCursor *cursor = source_object;
+	QueryRequest *request = task_data;
 	const gchar **variable_names = NULL;
+	GError *write_error = NULL;
 	gint i, n_columns;
 
-	if (!error) {
-		n_columns = tracker_sparql_cursor_get_n_columns (cursor);
-		variable_names = g_new0 (const gchar *, n_columns + 1);
-		for (i = 0; i < n_columns; i++)
-			variable_names[i] = tracker_sparql_cursor_get_variable_name (cursor, i);
+	n_columns = tracker_sparql_cursor_get_n_columns (cursor);
+	variable_names = g_new0 (const gchar *, n_columns + 1);
+	for (i = 0; i < n_columns; i++)
+		variable_names[i] = tracker_sparql_cursor_get_variable_name (cursor, i);
 
-		write_cursor (request, cursor, &error);
-	}
+	write_cursor (request, cursor, &write_error);
 
-	if (error)
-		g_dbus_method_invocation_return_gerror (request->invocation, error);
+	if (write_error)
+		g_dbus_method_invocation_return_gerror (request->invocation, write_error);
 	else
 		g_dbus_method_invocation_return_value (request->invocation, g_variant_new ("(^as)", variable_names));
 
 	g_free (variable_names);
-	g_clear_object (&cursor);
+	g_clear_error (&write_error);
+
+	g_task_return_boolean (task, TRUE);
+}
+
+static void
+finish_query (GObject      *source_object,
+              GAsyncResult *res,
+              gpointer      user_data)
+{
+	TrackerSparqlCursor *cursor = TRACKER_SPARQL_CURSOR (source_object);
+	GError *error = NULL;
+
+	if (!g_task_propagate_boolean (G_TASK (res), &error))
+		g_critical ("Error writing cursor: %s\n", error->message);
+
+	tracker_sparql_cursor_close (cursor);
+	g_object_unref (cursor);
+	g_clear_error (&error);
 }
 
 static void
@@ -297,11 +327,21 @@ query_cb (GObject      *object,
 	QueryRequest *request = user_data;
 	TrackerSparqlCursor *cursor;
 	GError *error = NULL;
+	GTask *task;
 
 	cursor = tracker_sparql_connection_query_finish (TRACKER_SPARQL_CONNECTION (object),
 	                                                 res, &error);
-	handle_cursor_reply (request, cursor, error);
-	query_request_free (request);
+	if (!cursor) {
+		g_dbus_method_invocation_return_gerror (request->invocation, error);
+		g_error_free (error);
+		query_request_free (request);
+		return;
+	}
+
+	task = g_task_new (cursor, request->endpoint->cancellable, finish_query, NULL);
+	g_task_set_task_data (task, request, (GDestroyNotify) query_request_free);
+	g_task_run_in_thread (task, handle_cursor_reply);
+	g_object_unref (task);
 }
 
 static void
@@ -312,11 +352,21 @@ stmt_execute_cb (GObject      *object,
 	QueryRequest *request = user_data;
 	TrackerSparqlCursor *cursor;
 	GError *error = NULL;
+	GTask *task;
 
 	cursor = tracker_sparql_statement_execute_finish (TRACKER_SPARQL_STATEMENT (object),
 	                                                  res, &error);
-	handle_cursor_reply (request, cursor, error);
-	query_request_free (request);
+	if (!cursor) {
+		g_dbus_method_invocation_return_gerror (request->invocation, error);
+		g_error_free (error);
+		query_request_free (request);
+		return;
+	}
+
+	task = g_task_new (cursor, request->endpoint->cancellable, finish_query, NULL);
+	g_task_set_task_data (task, request, (GDestroyNotify) query_request_free);
+	g_task_run_in_thread (task, handle_cursor_reply);
+	g_object_unref (task);
 }
 
 static void
@@ -359,7 +409,6 @@ read_update_cb (GObject      *object,
 		tracker_sparql_connection_update_array_async (conn,
 		                                              (gchar **) request->queries->pdata,
 		                                              request->queries->len,
-		                                              G_PRIORITY_DEFAULT,
 		                                              request->endpoint->cancellable,
 		                                              update_cb,
 		                                              request);
@@ -410,7 +459,6 @@ read_update_blank_cb (GObject      *object,
 	conn = tracker_endpoint_get_sparql_connection (TRACKER_ENDPOINT (request->endpoint));
 	tracker_sparql_connection_update_blank_async (conn,
 	                                              g_ptr_array_index (request->queries, 0),
-	                                              G_PRIORITY_DEFAULT,
 	                                              request->endpoint->cancellable,
 	                                              update_blank_cb,
 	                                              request);
@@ -520,6 +568,7 @@ endpoint_dbus_iface_method_call (GDBusConnection       *connection,
 			}
 		}
 
+		g_variant_iter_free (arguments);
 		g_free (query);
 	} else if (g_strcmp0 (method_name, "Update") == 0 ||
 	           g_strcmp0 (method_name, "UpdateArray") == 0) {
@@ -634,7 +683,7 @@ tracker_endpoint_dbus_initable_init (GInitable     *initable,
 		                                   error);
 
 	conn = tracker_endpoint_get_sparql_connection (endpoint);
-	endpoint_dbus->notifier = tracker_sparql_connection_create_notifier (conn, 0);
+	endpoint_dbus->notifier = tracker_sparql_connection_create_notifier (conn);
 	g_signal_connect (endpoint_dbus->notifier, "events",
 	                  G_CALLBACK (notifier_events_cb), endpoint);
 
