@@ -35,18 +35,6 @@ G_DECLARE_DERIVABLE_TYPE (TrackerNotifier, tracker_notifier, TRACKER, NOTIFIER,
 
 typedef struct _TrackerNotifierEvent TrackerNotifierEvent;
 
-/**
- * TrackerNotifierFlags:
- * @TRACKER_NOTIFIER_FLAG_NONE: No flags
- * @TRACKER_NOTIFIER_FLAG_QUERY_URN: Query URN of notified elements
- *
- * Flags affecting #TrackerNotifier behavior.
- */
-typedef enum {
-	TRACKER_NOTIFIER_FLAG_NONE               = 0,
-	TRACKER_NOTIFIER_FLAG_QUERY_URN          = 1 << 1,
-} TrackerNotifierFlags;
-
 /**
  * TrackerNotifierEventType:
  * @TRACKER_NOTIFIER_EVENT_CREATE: An element was created.
@@ -65,6 +53,7 @@ TRACKER_AVAILABLE_IN_ALL
 guint tracker_notifier_signal_subscribe   (TrackerNotifier *notifier,
                                            GDBusConnection *connection,
                                            const gchar     *service,
+                                           const gchar     *object_path,
                                            const gchar     *graph);
 TRACKER_AVAILABLE_IN_ALL
 void  tracker_notifier_signal_unsubscribe (TrackerNotifier *notifier,
