@@ -138,6 +138,7 @@ const TestInfo tests[] = {
 	{ "graph/graph-6", "graph/data-5", FALSE },
 	{ "graph/non-existent-1",  "graph/data-1", FALSE },
 	{ "graph/non-existent-2",  "graph/data-1", FALSE },
+	{ "graph/non-existent-3",  "graph/data-1", FALSE },
 	{ "graph/drop", "graph/data-drop", FALSE },
 	{ "graph/drop-non-existent", "graph/data-drop-non-existent", FALSE, TRUE },
 	{ "graph/drop-default", "graph/data-drop-default", FALSE },
@@ -194,6 +195,7 @@ const TestInfo tests[] = {
 	{ "subqueries/subqueries-1", "subqueries/data-1", FALSE },
 	{ "subqueries/subqueries-union-1", "subqueries/data-1", FALSE },
 	{ "subqueries/subqueries-union-2", "subqueries/data-1", FALSE },
+	{ "subqueries/subqueries-union-3", "subqueries/data-1", FALSE },
 	{ "subqueries/subqueries-minus-1", "subqueries/data-1", FALSE },
 	{ "subqueries/subqueries-minus-2", "subqueries/data-1", FALSE },
 	{ "subqueries/subqueries-minus-3", "subqueries/data-1", FALSE },
@@ -233,11 +235,17 @@ const TestInfo tests[] = {
 	{ "property-paths/mixed-inverse-and-sequence-2", "property-paths/data", FALSE },
 	{ "property-paths/mixed-inverse-and-sequence-3", "property-paths/data", FALSE },
 	{ "property-paths/mixed-recursive-and-sequence-1", "property-paths/data", FALSE },
+	{ "property-paths/mixed-recursive-and-sequence-2", "property-paths/data-2", FALSE },
+	{ "property-paths/mixed-recursive-and-sequence-3", "property-paths/data-2", FALSE },
+	{ "property-paths/mixed-recursive-and-sequence-4", "property-paths/data-2", FALSE },
+	{ "property-paths/mixed-recursive-and-sequence-5", "property-paths/data-2", FALSE },
 	{ "property-paths/mixed-recursive-and-alternative-1", "property-paths/data", FALSE },
 	{ "property-paths/mixed-recursive-and-alternative-2", "property-paths/data", FALSE },
 	{ "property-paths/mixed-recursive-and-inverse-1", "property-paths/data", FALSE },
 	{ "property-paths/mixed-recursive-and-inverse-2", "property-paths/data", FALSE },
 	{ "property-paths/mixed-recursive-and-inverse-3", "property-paths/data", FALSE },
+	{ "property-paths/mixed-optional-and-sequence-1", "property-paths/data-2", FALSE },
+	{ "property-paths/mixed-optional-and-sequence-2", "property-paths/data-2", FALSE },
 	/* Update tests */
 	{ "update/insert-data-query-1", "update/insert-data-1", FALSE, FALSE },
 	{ "update/insert-data-query-2", "update/insert-data-2", FALSE, TRUE },
@@ -300,8 +308,8 @@ check_result (TrackerDBCursor *cursor,
 				 * test reproduceability in DESCRIBE and other unrestricted
 				 * queries.
 				 */
-				if (g_strcmp0 (str, TRACKER_PREFIX_TRACKER "modified") == 0 ||
-				    g_strcmp0 (str, TRACKER_PREFIX_TRACKER "added") == 0) {
+				if (g_strcmp0 (str, TRACKER_PREFIX_NRL "modified") == 0 ||
+				    g_strcmp0 (str, TRACKER_PREFIX_NRL "added") == 0) {
 					g_string_free (row_str, TRUE);
 					row_str = NULL;
 					break;
